@@ -39,6 +39,10 @@
 
 #include "libbluray/bluray.h"
 
+extern "C" {
+#include <libavutil/rational.h>
+}
+
 static void stream_info(const std::string &path, BLURAY_STREAM_INFO *ss, int *channels, int *sample_rate, int *audio, int *width, int *height, AVRational *framerate, int *interleaved);
 static int parse_find_best_audio_stream();
 static int parse_find_best_video_stream();
@@ -379,7 +383,37 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
         // break;
     }
 
-    LPVIRTUALFILE virtualfile = insert_file(VIRTUALTYPE_BLURAY, path + filename, &stbuf);
+    LPVIRTUALFILE virtualfile = nullptr;
+    if (!params.m_format[0].is_multiformat())
+    {
+        virtualfile = insert_file(VIRTUALTYPE_BLURAY, path + filename, &stbuf);
+    }
+    else
+    {
+        std::string origpath(path + filename);
+
+        int flags = VIRTUALFLAG_DIRECTORY | VIRTUALFLAG_FILESET;
+
+        // Change file to directory for the frame set
+        // Change file to virtual directory for the frame set. Keep permissions.
+        stbuf.st_mode  &= ~static_cast<mode_t>(S_IFREG | S_IFLNK);
+        stbuf.st_mode  |= S_IFDIR;
+        stbuf.st_nlink = 2;
+        stbuf.st_size  = stbuf.st_blksize;
+
+        append_sep(&origpath);
+
+        if (params.m_format[0].is_frameset())
+        {
+            flags |= VIRTUALFLAG_FRAME;
+        }
+        else if (params.m_format[0].is_hls())
+        {
+            flags |= VIRTUALFLAG_HLS;
+        }
+
+        virtualfile = insert_file(VIRTUALTYPE_BLURAY, origpath, &stbuf, flags);
+    }
 
     // Bluray is video format anyway
     virtualfile->m_format_idx           = 0;
@@ -432,13 +466,16 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
             stream_info(path, &clip->video_streams[parse_find_best_video_stream()], &channels, &sample_rate, &audio, &width, &height, &framerate, &interleaved);
         }
 
-        Logging::debug(virtualfile->m_origfile, "Video %1 %2x%3@%<%5.2f>4%5 fps %6 [%7]", format_bitrate(video_bit_rate).c_str(), width, height, av_q2d(framerate), interleaved ? "i" : "p", format_size(size).c_str(), format_duration(duration).c_str());
+        Logging::trace(virtualfile->m_origfile, "Video %1 %2x%3@%<%5.2f>4%5 fps %6 [%7]", format_bitrate(video_bit_rate).c_str(), width, height, av_q2d(framerate), interleaved ? "i" : "p", format_size(size).c_str(), format_duration(duration).c_str());
         if (audio > -1)
         {
-            Logging::debug(virtualfile->m_origfile, "Audio %1 channels %2", channels, format_samplerate(sample_rate).c_str());
+            Logging::trace(virtualfile->m_origfile, "Audio %1 channels %2", channels, format_samplerate(sample_rate).c_str());
         }
 
         transcoder_set_filesize(virtualfile, duration, audio_bit_rate, channels, sample_rate, video_bit_rate, width, height, interleaved, framerate);
+
+        virtualfile->m_video_frame_count    = static_cast<uint32_t>(av_rescale_q(duration, av_get_time_base_q(), av_inv_q(framerate)));
+        virtualfile->m_predicted_size       = size;
     }
 
     return true;
