@@ -38,6 +38,29 @@
 
 #include <map>
 #include <sqlite3.h>
+
+#define     DB_BASE_VERSION_MAJOR   1           /**< brief The oldest database version major (Release < 1.95) */
+#define     DB_BASE_VERSION_MINOR   0           /**< brief The oldest database version minor (Release < 1.95) */
+
+#define     DB_VERSION_MAJOR        1           /**< brief Current database version major */
+#define     DB_VERSION_MINOR        97          /**< brief Current database version minor */
+
+#define     DB_MIN_VERSION_MAJOR    1           /**< brief Required database version major (required 1.95) */
+#define     DB_MIN_VERSION_MINOR    97          /**< brief Required database version minor (required 1.95) */
+
+/**
+  * @brief RESULTCODE of transcoding operation
+  */
+typedef enum RESULTCODE
+{
+    RESULTCODE_NONE,                            /**< @brief No result code available */
+    RESULTCODE_INCOMPLETE,                      /**< @brief Transcode finished, but incomplete */
+    RESULTCODE_FINISHED,                        /**< @brief Transcode finished successfully */
+    RESULTCODE_ERROR,                           /**< @brief Transcode finished with error */
+} RESULTCODE;
+typedef RESULTCODE const *LPCRESULTCODE;        /**< @brief Pointer version of RESULTCODE */
+typedef RESULTCODE *LPRESULTCODE;               /**< @brief Pointer to const version of RESULTCODE */
+
 /**
   * @brief Cache information block
   */
@@ -52,9 +75,12 @@ typedef struct CACHE_INFO
     int             m_videowidth;               /**< @brief Video width */
     int             m_videoheight;              /**< @brief Video height */
     bool            m_deinterlace;              /**< @brief true if video was deinterlaced */
+    int64_t         m_duration;                 /**< @brief File  duration, in AV_TIME_BASE fractional seconds. */
     size_t          m_predicted_filesize;       /**< @brief Predicted file size */
     size_t          m_encoded_filesize;         /**< @brief Actual file size after encode */
-    bool            m_finished;                 /**< @brief true if decode has finished */
+    uint32_t        m_video_frame_count;        /**< @brief Number of frames in video or 0 if not a video */
+    uint32_t        m_segment_count;            /**< @brief Number of video segments for HLS */
+    RESULTCODE      m_finished;                 /**< @brief Result code: */
     bool            m_error;                    /**< @brief true if encode failed */
     int             m_errno;                    /**< @brief errno if encode failed */
     int             m_averror;                  /**< @brief FFmpeg error code if encode failed */
@@ -76,6 +102,28 @@ class Cache
 {
     typedef std::pair<std::string, std::string> cache_key_t;
     typedef std::map<cache_key_t, Cache_Entry *> cache_t;
+public:
+    /**
+      * @brief Definition of sql table
+      */
+    typedef struct
+    {
+        const char *    name;                       /**< @brief Table name */
+        const char *    primary_key;                /**< @brief Primary key of table */
+    } TABLE_DEF;
+    typedef TABLE_DEF const *LPCTABLE_DEF;          /**< @brief Pointer version of TABLE_DEF */
+    typedef TABLE_DEF *LPTABLE_DEF;                 /**< @brief Pointer to const version of TABLE_DEF */
+
+    /**
+      * @brief Column definition of sql table
+      */
+    typedef struct
+    {
+        const char *    name;                       /**< @brief Column name */
+        const char *    type;                       /**< @brief Column type (INT, CHAR etc) */
+    } TABLE_COLUMNS;
+    typedef TABLE_COLUMNS const *LPCTABLE_COLUMNS;  /**< @brief Pointer version of TABLE_COLUMNS */
+    typedef TABLE_COLUMNS *LPTABLE_COLUMNS;         /**< @brief Pointer to const version of TABLE_COLUMNS */
 
     friend class Cache_Entry;
 
@@ -201,8 +249,81 @@ protected:
      * @return Returns the SQL string bound to the statement handle.
      */
     std::string             expanded_sql(sqlite3_stmt *pStmt);
+    /**
+     * @brief Prepare all SQL statements
+     * @return Returns true on success, false on error.
+     */
+    bool                    prepare_stmts();
+    /**
+     * @brief Check if SQL table exists in database.
+     * @param[in] table - name of table
+     * @return Returns true if table exists, false if not.
+     */
+    bool                    table_exists(const char *table);
+    /**
+     * @brief Check if column exists in SQL table.
+     * @param[in] table - name of table
+     * @param[in] column - name of column
+     * @return Returns true if table exists, false if not.
+     */
+    bool                    column_exists(const char *table, const char *column);
+    /**
+     * @brief Check the db version if upgrade needed.
+     * @param[out] db_version_major - Upon return, contains the major database version.
+     * @param[out] db_version_minor - Upon return, contains the minor database version.
+     * @return Returns true of version is OK, false if upgrade is needed.
+     */
+    bool                    check_min_version(int *db_version_major, int *db_version_minor);
+    /**
+     * @brief Compare two versions.
+     * @param[in] version_major_l - Left major version
+     * @param[in] version_minor_l - Left minor version
+     * @param[in] version_major_r - Right major version
+     * @param[in] version_minor_r - Right minor version
+     * @return Returns +1 if left version is larger then right, 0 if versions are the same, -1 if right version is larger than left.
+     */
+    int                     cmp_version(int version_major_l, int version_minor_l, int version_major_r, int version_minor_r);
+    /**
+     * @brief Begin a database transactio.n
+     * @return Returns true on success; false on error.
+     */
+    bool                    begin_transaction();
+    /**
+     * @brief End a database transaction.
+     * @return Returns true on success; false on error.
+     */
+    bool                    end_transaction();
+    /**
+     * @brief Rollback a database transaction.
+     * @return Returns true on success; false on error.
+     */
+    bool                    rollback_transaction();
+    /**
+     * @brief Create cache_entry table
+     * @return
+     * @return Returns true on success; false on error.
+     */
+    bool                    create_table_cache_entry(LPCTABLE_DEF table, const TABLE_COLUMNS columns[]);
+    /*
+     */
+    /**
+     * @brief Upgrade database from version below 1.95
+     * @param[out] db_version_major - Upon return, contains the new major database version.
+     * @param[out] db_version_minor - Upon return, contains the new minor database version.
+     * @return Returns true on success; false on error.
+     */
+    bool                    upgrade_db(int *db_version_major, int *db_version_minor);
 
 private:
+    static const
+    TABLE_DEF               m_table_cache_entry;
+    static const
+    TABLE_COLUMNS           m_columns_cache_entry[];
+    static const
+    TABLE_DEF               m_table_version;
+    static const
+    TABLE_COLUMNS           m_columns_version[];
+
     std::recursive_mutex    m_mutex;                        /**< @brief Access mutex */
     std::string             m_cacheidx_file;                /**< @brief Name of SQLite cache index database */
     sqlite3*                m_cacheidx_db;                  /**< @brief SQLite handle of cache index database */
