@@ -40,6 +40,7 @@ Cache_Entry::Cache_Entry(Cache *owner, LPVIRTUALFILE virtualfile)
     : m_owner(owner)
     , m_ref_count(0)
     , m_virtualfile(virtualfile)
+    , m_seek_to_no(0)
 {
     m_cache_info.m_origfile = virtualfile->m_origfile;
 
@@ -57,7 +58,7 @@ Cache_Entry::Cache_Entry(Cache *owner, LPVIRTUALFILE virtualfile)
 
     clear();
 
-    Logging::debug(filename(), "Created new cache entry.");
+    Logging::trace(filename(), "Created new cache entry.");
 }
 
 Cache_Entry::~Cache_Entry()
@@ -98,14 +99,11 @@ void Cache_Entry::clear(bool fetch_file_time /*= true*/)
     m_cache_info.m_videobitrate         = params.m_videobitrate;
     m_cache_info.m_videowidth           = params.m_videowidth;
     m_cache_info.m_videoheight          = params.m_videoheight;
-#ifndef USING_LIBAV
     m_cache_info.m_deinterlace          = params.m_deinterlace;
-#else   // !USING_LIBAV
-    m_cache_info.m_deinterlace          = 0;
-#endif  // USING_LIBAV
     m_cache_info.m_predicted_filesize   = 0;
     m_cache_info.m_encoded_filesize     = 0;
-    m_cache_info.m_finished             = false;
+    m_cache_info.m_video_frame_count    = 0;
+    m_cache_info.m_finished             = RESULTCODE_NONE;
     m_cache_info.m_error                = false;
     m_cache_info.m_errno                = 0;
     m_cache_info.m_averror              = 0;
@@ -183,7 +181,7 @@ bool Cache_Entry::open(bool create_cache /*= true*/)
         return true;
     }
 
-    if (!m_cache_info.m_finished)
+    if (m_cache_info.m_finished != RESULTCODE_INCOMPLETE && m_cache_info.m_finished != RESULTCODE_FINISHED)
     {
         // If no database entry found (database is not consistent),
         // or file was not completely transcoded last time,
@@ -283,6 +281,11 @@ size_t Cache_Entry::size() const
     }
 }
 
+uint32_t Cache_Entry::video_frame_count() const
+{
+    return m_cache_info.m_video_frame_count;
+}
+
 time_t Cache_Entry::age() const
 {
     return (time(nullptr) - m_cache_info.m_creation_time);
@@ -361,13 +364,11 @@ bool Cache_Entry::outdated() const
         return true;
     }
 
-#ifndef USING_LIBAV
     if (m_cache_info.m_deinterlace != params.m_deinterlace)
     {
         Logging::debug(filename(), "Triggering re-transcode: Selected video deinterlace changed from %1 to %2.", m_cache_info.m_deinterlace, params.m_deinterlace);
         return true;
     }
-#endif  // !USING_LIBAV
 
     if (stat(filename().c_str(), &sb) != -1)
     {
