@@ -231,15 +231,14 @@ void FFmpeg_Base::video_stream_setup(AVCodecContext *output_codec_ctx, AVStream*
     output_stream->codec->time_base             = time_base_tbc;
 #endif
 
-#ifndef USING_LIBAV
     // tbr
     // output_stream->r_frame_rate              = m_in.m_pVideo_stream->r_frame_rate;
-    // output_stream->r_frame_rate              = { .num = 25, .den = 1 };
+    output_stream->r_frame_rate                 = framerate;
 
     // fps
     output_stream->avg_frame_rate               = framerate;
-    output_codec_ctx->framerate                 = framerate;
-#endif
+    // output_codec_ctx->framerate                 = framerate;
+
     int alpha = 0;
     int loss = 0;
 
@@ -247,12 +246,7 @@ void FFmpeg_Base::video_stream_setup(AVCodecContext *output_codec_ctx, AVStream*
     AVPixelFormat  dst_pix_fmt                  = AV_PIX_FMT_NONE;
     if (output_codec_ctx->codec->pix_fmts != nullptr)
     {
-#ifndef USING_LIBAV
         dst_pix_fmt = avcodec_find_best_pix_fmt_of_list(output_codec_ctx->codec->pix_fmts, src_pix_fmt, alpha, &loss);
-#else
-        // Yes, casting const away is bad. Libav is bad, too. OK, two wrongs make no right. To fix that use FFmpeg. Don't use libav. Did I mention it's bad?
-        dst_pix_fmt = avcodec_find_best_pix_fmt2(const_cast<AVPixelFormat *>(output_codec_ctx->codec->pix_fmts), src_pix_fmt, alpha, &loss);
-#endif
     }
 
     if (dst_pix_fmt == AV_PIX_FMT_NONE)
@@ -352,12 +346,24 @@ std::string FFmpeg_Base::get_sample_fmt_name(AVSampleFormat sample_fmt) const
 std::string FFmpeg_Base::get_channel_layout_name(int nb_channels, uint64_t channel_layout) const
 {
     char buffer[1024];
-#ifndef USING_LIBAV
     av_get_channel_layout_string(buffer, sizeof(buffer), nb_channels, channel_layout);
-#else
-    // av_get_channel_layout_string not supported by Libav
-    std::snprintf(buffer, sizeof(buffer) - 1, "%" PRId64, channel_layout);
-#endif
     return buffer;
 }
 
+uint32_t FFmpeg_Base::pts_to_frame(AVStream* stream, int64_t pts) const
+{
+    if (pts == AV_NOPTS_VALUE)
+    {
+        return 0;
+    }
+    int64_t start_time = (stream->start_time != AV_NOPTS_VALUE) ? stream->start_time : 0;
+    AVRational factor = av_mul_q(stream->r_frame_rate, stream->time_base);
+    return static_cast<uint32_t>(av_rescale(pts - start_time, factor.num, factor.den) + 1);
+}
+
+int64_t FFmpeg_Base::frame_to_pts(AVStream* stream, uint32_t frame_no) const
+{
+    int64_t start_time = (stream->start_time != AV_NOPTS_VALUE) ? stream->start_time : 0;
+    AVRational factor = av_mul_q(stream->r_frame_rate, stream->time_base);
+    return static_cast<uint32_t>(av_rescale(frame_no - 1, factor.den, factor.num) + start_time);
+}
