@@ -65,13 +65,7 @@ extern "C" {
 #else
 #include <libavresample/avresample.h>
 #endif
-#ifndef USING_LIBAV
-// Does not exist in Libav
 #include "libavutil/ffversion.h"
-#else
-#include "libavutil/avstring.h"
-#include "libavutil/opt.h"
-#endif
 #ifdef __cplusplus
 }
 #endif
@@ -195,6 +189,43 @@ bool FFmpegfs_Format::init(const std::string & desttype)
         m_fileext           = "m4a";
         break;
     }
+    case FILETYPE_PNG:
+    {
+        m_audio_codec_id    = AV_CODEC_ID_NONE;
+        m_video_codec_id    = AV_CODEC_ID_PNG;
+        m_format_name       = "png";
+        m_fileext           = "png";
+        break;
+    }
+    case FILETYPE_JPG:
+    {
+        m_audio_codec_id    = AV_CODEC_ID_NONE;
+        m_video_codec_id    = AV_CODEC_ID_MJPEG;
+        m_format_name       = "jpg";
+        m_fileext           = "jpg";
+        break;
+    }
+    case FILETYPE_BMP:
+    {
+        m_audio_codec_id    = AV_CODEC_ID_NONE;
+        m_video_codec_id    = AV_CODEC_ID_BMP;
+        m_format_name       = "bmp";
+        m_fileext           = "bmp";
+        break;
+    }
+    case FILETYPE_TS:
+    case FILETYPE_HLS:
+    {
+        // AC3 possible in container, but not supported in browsers:
+        // m_audio_codec_id    = AV_CODEC_ID_AC3;
+        // Also allowed:
+        // m_audio_codec_id    = AV_CODEC_ID_MP3;
+        m_audio_codec_id    = AV_CODEC_ID_AAC;
+        m_video_codec_id    = AV_CODEC_ID_H264;
+        m_format_name       = "mpegts";
+        m_fileext           = "ts";
+        break;
+    }
     case FILETYPE_UNKNOWN:
     {
         found = false;
@@ -225,6 +256,21 @@ FILETYPE FFmpegfs_Format::filetype() const
     return m_filetype;
 }
 
+bool FFmpegfs_Format::is_multiformat() const
+{
+    return  (is_frameset() || is_hls());
+}
+
+bool FFmpegfs_Format::is_frameset() const
+{
+    return (m_filetype == FILETYPE_JPG || m_filetype == FILETYPE_PNG || m_filetype == FILETYPE_BMP);
+}
+
+bool FFmpegfs_Format::is_hls() const
+{
+    return (m_filetype == FILETYPE_HLS);
+}
+
 AVCodecID FFmpegfs_Format::video_codec_id() const
 {
     return m_video_codec_id;
@@ -351,6 +397,21 @@ const std::string & replace_ext(std::string * filepath, const std::string & ext)
     return *filepath;
 }
 
+const std::string & append_ext(std::string * filepath, const std::string & ext)
+{
+    size_t found;
+
+    found = filepath->rfind('.');
+
+    if (found == std::string::npos || strcasecmp(filepath->substr(found + 1), ext))
+    {
+        // No extension or different extension
+        *filepath += '.' + ext;
+    }
+
+    return *filepath;
+}
+
 char * new_strdup(const std::string & str)
 {
     size_t n = str.size() + 1;
@@ -378,9 +439,16 @@ const std::string & get_destname(std::string *destfilepath, const std::string &
 
 std::string ffmpeg_geterror(int errnum)
 {
-    char error[AV_ERROR_MAX_STRING_SIZE];
-    av_strerror(errnum, error, AV_ERROR_MAX_STRING_SIZE);
-    return error;
+    if (errnum < 0)
+    {
+        char error[AV_ERROR_MAX_STRING_SIZE];
+        av_strerror(errnum, error, AV_ERROR_MAX_STRING_SIZE);
+        return error;
+    }
+    else
+    {
+        return strerror(errnum);
+    }
 }
 
 int64_t ffmpeg_rescale(int64_t ts, const AVRational & time_base)
@@ -455,11 +523,7 @@ std::string ffmpeg_libinfo()
 {
     std::string info;
 
-#ifdef USING_LIBAV
-    info = "Libav Version       :\n";
-#else
     info = "FFmpeg Version      : " FFMPEG_VERSION "\n";
-#endif
 
     info += PRINT_LIB_INFO(avutil,      AVUTIL);
     info += PRINT_LIB_INFO(avcodec,     AVCODEC);
@@ -686,6 +750,11 @@ FILETYPE get_filetype(const std::string & desttype)
         { "opus",   FILETYPE_OPUS },
         { "prores", FILETYPE_PRORES },
         { "alac",   FILETYPE_ALAC },
+        { "png",    FILETYPE_PNG },
+        { "jpg",    FILETYPE_JPG },
+        { "bmp",    FILETYPE_BMP },
+        { "ts",     FILETYPE_TS },
+        { "hls",    FILETYPE_HLS },
     };
 
     try
@@ -712,88 +781,6 @@ FILETYPE get_filetype_from_list(const std::string & desttypelist)
     return filetype;
 }
 
-#ifdef USING_LIBAV
-int avformat_alloc_output_context2(AVFormatContext **avctx, AVOutputFormat *oformat, const char *format, const char *filename)
-{
-    AVFormatContext *s = avformat_alloc_context();
-    int ret = 0;
-
-    *avctx = nullptr;
-    if (s == nullptr)
-        goto nomem;
-
-    if (oformat == nullptr)
-    {
-        if (format != nullptr)
-        {
-            oformat = av_guess_format(format, nullptr, nullptr);
-            if (oformat == nullptr)
-            {
-                av_log(s, AV_LOG_ERROR, "Requested output format '%s' is not a suitable output format\n", format);
-                ret = AVERROR(EINVAL);
-                goto error;
-            }
-        }
-        else
-        {
-            oformat = av_guess_format(nullptr, filename, nullptr);
-            if (oformat == nullptr)
-            {
-                ret = AVERROR(EINVAL);
-                av_log(s, AV_LOG_ERROR, "%s * Unable to find a suitable output format\n", filename);
-                goto error;
-            }
-        }
-    }
-
-    s->oformat = oformat;
-    if (s->oformat->priv_data_size > 0)
-    {
-        s->priv_data = av_mallocz(s->oformat->priv_data_size);
-        if (s->priv_data == nullptr)
-            goto nomem;
-        if (s->oformat->priv_class != nullptr)
-        {
-            *(const AVClass**)s->priv_data= s->oformat->priv_class;
-            av_opt_set_defaults(s->priv_data);
-        }
-    }
-    else
-        s->priv_data = nullptr;
-
-    if (filename)
-        av_strlcpy(s->filename, filename, sizeof(s->filename));
-    *avctx = s;
-    return 0;
-nomem:
-    av_log(s, AV_LOG_ERROR, "Out of memory\n");
-    ret = AVERROR(ENOMEM);
-error:
-    avformat_free_context(s);
-    return ret;
-}
-
-const char *avcodec_get_name(enum AVCodecID id)
-{
-    const AVCodecDescriptor *cd;
-    AVCodec *codec;
-
-    if (id == AV_CODEC_ID_NONE)
-        return "none";
-    cd = avcodec_descriptor_get(id);
-    if (cd)
-        return cd->name;
-    av_log(nullptr, AV_LOG_WARNING, "Codec 0x%x is not in the full list.\n", id);
-    codec = avcodec_find_decoder(id);
-    if (codec)
-        return codec->name;
-    codec = avcodec_find_encoder(id);
-    if (codec)
-        return codec->name;
-    return "unknown_codec";
-}
-#endif
-
 void init_id3v1(ID3v1 *id3v1)
 {
     // Initialise ID3v1.1 tag structure
@@ -1060,18 +1047,14 @@ int print_stream_info(const AVStream* stream)
     AVCodecContext *avctx = stream->codec;
 #endif
     int fps = stream->avg_frame_rate.den && stream->avg_frame_rate.num;
-#ifndef USING_LIBAV
     int tbr = stream->r_frame_rate.den && stream->r_frame_rate.num;
-#endif
     int tbn = stream->time_base.den && stream->time_base.num;
     int tbc = avctx->time_base.den && avctx->time_base.num; // Even the currently latest (lavf 58.10.100) refers to AVStream codec->time_base member... (See dump.c dump_stream_format)
 
     if (fps)
         print_fps(av_q2d(stream->avg_frame_rate), "avg fps");
-#ifndef USING_LIBAV
     if (tbr)
         print_fps(av_q2d(stream->r_frame_rate), "Real base framerate (tbr)");
-#endif
     if (tbn)
         print_fps(1 / av_q2d(stream->time_base), "stream timebase (tbn)");
     if (tbc)
@@ -1265,10 +1248,10 @@ std::string sanitise_filepath(std::string * filepath)
     if (realpath(filepath->c_str(), resolved_name) != nullptr)
     {
         *filepath = resolved_name;
-    	return *filepath;
+        return *filepath;
     }
 
-	// realpath has the strange feature to remove a traling slash if there.
+    // realpath has the strange feature to remove a traling slash if there.
     // To mimick its behaviour, if realpath fails, at least remove it.
     std::string _filepath(*filepath);
     remove_sep(&_filepath);
@@ -1326,3 +1309,12 @@ bool check_ignore(size_t size, size_t offset)
     return ignore;
 }
 
+std::string make_filename(uint32_t file_no, const std::string & fileext)
+{
+    return string_format("%06u.%s", file_no, fileext.c_str());
+}
+
+bool file_exists(const std::string & filename)
+{
+    return (access(filename.c_str(), F_OK) != -1);
+}
