@@ -87,14 +87,6 @@ extern "C" {
 #error "LIBAVUTIL_VERSION_MICRO not defined. Missing include header?"
 #endif
 
-// Libav detection:
-// FFmpeg has library micro version >= 100
-// Libav  has library micro version < 100
-// So if micro < 100, it's Libav, else it's FFmpeg.
-#if LIBAVUTIL_VERSION_MICRO < 100
-#define USING_LIBAV                         /**< @brief If defined, we are linking to Libav */
-#endif
-
 /**
  * Allow use of av_format_inject_global_side_data when available
  */
@@ -150,14 +142,14 @@ const char *get_media_type_string(enum AVMediaType media_type);
 #define AV_CODEC_CAP_VARIABLE_FRAME_SIZE	CODEC_CAP_VARIABLE_FRAME_SIZE           /**< @brief AV_CODEC_CAP_VARIABLE_FRAME_SIZE is missing in older FFmpeg versions */
 #endif
 
-#if !defined(USING_LIBAV) && (LIBAVUTIL_VERSION_MAJOR > 54)
+#if (LIBAVUTIL_VERSION_MAJOR > 54)
 #define BITRATE int64_t                                                             /**< @brief For FFmpeg bit rate is an int64. */
-#else // USING_LIBAV
+#else
 #define BITRATE int                                                                 /**< @brief For FFmpeg bit rate is an int. */
 #endif
 
 // Make access possible over codecpar if available
-#if LAVF_DEP_AVSTREAM_CODEC || defined(USING_LIBAV)
+#if LAVF_DEP_AVSTREAM_CODEC
 #define CODECPAR(s)     ((s)->codecpar)                                             /**< @brief Map to codecpar */
 #else
 #define CODECPAR(s)     ((s)->codec)                                                /**< @brief Map to codec */
@@ -181,6 +173,11 @@ typedef enum FILETYPE
     FILETYPE_OPUS,
     FILETYPE_PRORES,
     FILETYPE_ALAC,
+    FILETYPE_PNG,
+    FILETYPE_JPG,
+    FILETYPE_BMP,
+    FILETYPE_TS,
+    FILETYPE_HLS
 } FILETYPE;
 
 /**
@@ -239,6 +236,15 @@ typedef enum AUTOCOPY
     AUTOCOPY_STRICTLIMIT,  /**< @brief Same as STRICT, only copy if target not larger, transcode otherwise. */
 } AUTOCOPY;
 
+/**
+  * Recode to same format options
+  */
+typedef enum RECODESAME
+{
+     RECODESAME_NO = 0,     /**< @brief Never recode to same format. */
+     RECODESAME_YES,        /**< @brief Always recode to same format. */
+} RECODESAME;
+
 /**
  * @brief The #FFmpegfs_Format class
  */
@@ -296,11 +302,27 @@ public:
      * @return Returns audio codec_id
      */
     AVCodecID           audio_codec_id() const;
+    /**
+     * @brief Check if this si some sort of multi file format
+     * (any of the following: is_frameset() or is_hls()).
+     * @return Returns true for a multi file format.
+     */
+    bool                is_multiformat() const;
+    /**
+     * @brief Check for an export frame format
+     * @return Returns true for formats that export all frames as images.
+     */
+    bool                is_frameset() const;
+    /**
+     * @brief Check for HLS format
+     * @return Returns true for formats that create an HLS set including the m3u file.
+     */
+    bool                is_hls() const;
 
 protected:
     std::string m_format_name;              /**< @brief Descriptive name of the format, e.g. "Opus Audio". */
     std::string m_desttype;                 /**< @brief Destination type: mp4, mp3 or other */
-    std::string m_fileext;                   /**< @brief File extensio: mp4, mp3, flac or other */
+    std::string m_fileext;                  /**< @brief File extension: mp4, mp3, flac or other */
     FILETYPE    m_filetype;                 /**< @brief File type, MP3, MP4, OPUS etc. */
     AVCodecID   m_video_codec_id;           /**< @brief AVCodec used for video encoding */
     AVCodecID   m_audio_codec_id;           /**< @brief AVCodec used for audio encoding */
@@ -352,11 +374,18 @@ const std::string & remove_ext(std::string *filepath);
 bool                find_ext(std::string * ext, const std::string & filename);
 /**
  * @brief Replace extension in filename, taking into account that there might not be an extension already.
+ * @param[in] filepath - Filename to replace extension.
+ * @param[in] ext - Extension to replace.
+ * @return Returns constant reference to filename.
+ */
+const std::string & replace_ext(std::string * filepath, const std::string & ext);
+/**
+ * @brief Append extension to filename. If ext is the same as
  * @param[in] filepath - Filename to add extension to.
  * @param[in] ext - Extension to add.
  * @return Returns constant reference to filename.
  */
-const std::string & replace_ext(std::string * filepath, const std::string & ext);
+const std::string & append_ext(std::string * filepath, const std::string & ext);
 /**
  * @brief strdup() variant taking a std::string as input.
  * @param[in] str - String to duplicate.
@@ -564,35 +593,6 @@ int                 mktree(const std::string & path, mode_t mode);
  */
 void                tempdir(std::string & path);
 
-#ifdef USING_LIBAV
-/**
- * @brief Implement missing avformat_alloc_output_context2() for Libav.
- * @param[in] avctx - See FFmpeg avformat_alloc_output_context2() function.
- * @param[in] oformat - See FFmpeg avformat_alloc_output_context2() function.
- * @param[in] format - See FFmpeg avformat_alloc_output_context2() function.
- * @param[in] filename - See FFmpeg avformat_alloc_output_context2() function.
- * @return See FFmpeg avformat_alloc_output_context2() function.
- */
-int                 avformat_alloc_output_context2(AVFormatContext **avctx, AVOutputFormat *oformat, const char *format, const char *filename);
-/**
- * @brief Implement missing avcodec_get_name for Libav.
- * @param[in] id - See FFmpeg avformat_alloc_output_context2() function.
- * @return See FFmpeg avformat_alloc_output_context2() function.
- */
-const char *        avcodec_get_name(AVCodecID id);
-
-/**
- * Create a rational.
- * Useful for compilers that do not support compound literals.
- * @note  The return value is not reduced.
- */
-static inline AVRational av_make_q(int num, int den)
-{
-    AVRational r = { num, den };
-    return r;
-}
-#endif
-
 /**
  * @brief Split string into an array delimited by a regular expression.
  * @param[in] input - Input string.
@@ -671,4 +671,19 @@ size_t              get_disk_free(std::string & path);
  */
 bool                check_ignore(size_t size, size_t offset);
 
+/**
+ * @brief Make a file name from file number and file extension.
+ * @param[in] file_no - File number 1...n
+ * @param[in] fileext - Extension of file (e.g mp4, webm)
+ * @return Returns the file name.
+ */
+std::string         make_filename(uint32_t file_no, const std::string &fileext);
+
+/**
+ * @brief Check if file exists.
+ * @param[in] filename - File to check.
+ * @return Returns true if file exists, false if not.
+ */
+bool                file_exists(const std::string & filename);
+
 #endif
