@@ -87,13 +87,13 @@ FFMPEGFS_PARAMS::FFMPEGFS_PARAMS()
     // Format
     , m_audiobitrate(128*1024)                  // default: 128 kBit
     , m_audiosamplerate(44100)                  // default: 44.1 kHz
+    , m_audiochannels(2)                        // default: 2 channels
 
     , m_videobitrate(2*1024*1024)               // default: 2 MBit
     , m_videowidth(0)                           // default: do not change width
     , m_videoheight(0)                          // default: do not change height
-    #ifndef USING_LIBAV
     , m_deinterlace(0)                          // default: do not interlace video
-    #endif  // !USING_LIBAV
+    , m_segment_duration(10 * AV_TIME_BASE)     // default: 10 seconds
     // Album arts
     , m_noalbumarts(0)                          // default: copy album arts
     // Virtual Script
@@ -113,7 +113,7 @@ FFMPEGFS_PARAMS::FFMPEGFS_PARAMS()
     , m_prebuffer_size(100 /* KB */ * 1024)     // default: 100 KB
     , m_max_cache_size(0)                       // default: no limit
     , m_min_diskspace(0)                        // default: no minimum
-    , m_cachepath("")                           // default: /var/cache/ffmpegfs
+    , m_cachepath("")                           // default: $XDG_CACHE_HOME/ffmpegfs
     , m_disable_cache(0)                        // default: enabled
     , m_cache_maintenance((60*60))              // default: prune every 60 minutes
     , m_prune_cache(0)                          // default: Do not prune cache immediately
@@ -121,6 +121,7 @@ FFMPEGFS_PARAMS::FFMPEGFS_PARAMS()
     , m_max_threads(0)                          // default: 16 * CPU cores (this value here is overwritten later)
     , m_decoding_errors(0)                      // default: ignore errors
     , m_min_dvd_chapter_duration(1)             // default: 1 second
+    , m_oldnamescheme(0)                        // default: new scheme
     , m_win_smb_fix(0)                          // default: no fix
 {
 }
@@ -200,7 +201,9 @@ enum
     KEY_DESTTYPE,
     KEY_AUDIO_BITRATE,
     KEY_AUDIO_SAMPLERATE,
+    KEY_AUDIO_CHANNELS,
     KEY_VIDEO_BITRATE,
+    KEY_SEGMENT_DURATION,
     KEY_SCRIPTFILE,
     KEY_SCRIPTSOURCE,
     KEY_EXPIRY_TIME,
@@ -212,6 +215,7 @@ enum
     KEY_CACHEPATH,
     KEY_CACHE_MAINTENANCE,
     KEY_AUTOCOPY,
+    KEY_RECODESAME,
     KEY_PROFILE,
     KEY_LEVEL,
     KEY_LOG_MAXLEVEL,
@@ -235,6 +239,8 @@ static struct fuse_opt ffmpegfs_opts[] =
     FUSE_OPT_KEY("profile=%s",                      KEY_PROFILE),
     FUSE_OPT_KEY("--autocopy=%s",                   KEY_AUTOCOPY),
     FUSE_OPT_KEY("autocopy=%s",                     KEY_AUTOCOPY),
+    FUSE_OPT_KEY("--recodesame=%s",                 KEY_RECODESAME),
+    FUSE_OPT_KEY("recodesame=%s",                   KEY_RECODESAME),
     FUSE_OPT_KEY("--level=%s",                      KEY_LEVEL),
     FUSE_OPT_KEY("level=%s",                        KEY_LEVEL),
 
@@ -243,6 +249,8 @@ static struct fuse_opt ffmpegfs_opts[] =
     FUSE_OPT_KEY("audiobitrate=%s",                 KEY_AUDIO_BITRATE),
     FUSE_OPT_KEY("--audiosamplerate=%s",            KEY_AUDIO_SAMPLERATE),
     FUSE_OPT_KEY("audiosamplerate=%s",              KEY_AUDIO_SAMPLERATE),
+    FUSE_OPT_KEY("--audiochannels=%s",              KEY_AUDIO_CHANNELS),
+    FUSE_OPT_KEY("audiochannels=%s",                KEY_AUDIO_CHANNELS),
 
     // Video
     FUSE_OPT_KEY("--videobitrate=%s",               KEY_VIDEO_BITRATE),
@@ -251,10 +259,11 @@ static struct fuse_opt ffmpegfs_opts[] =
     FFMPEGFS_OPT("videoheight=%u",                  m_videoheight, 0),
     FFMPEGFS_OPT("--videowidth=%u",                 m_videowidth, 0),
     FFMPEGFS_OPT("videowidth=%u",                   m_videowidth, 0),
-#ifndef USING_LIBAV
     FFMPEGFS_OPT("--deinterlace",                   m_deinterlace, 1),
     FFMPEGFS_OPT("deinterlace",                     m_deinterlace, 1),
-#endif  // !USING_LIBAV
+    // HLS
+    FUSE_OPT_KEY("--segment_duration=%s",           KEY_SEGMENT_DURATION),
+    FUSE_OPT_KEY("segment_duration=%s",             KEY_SEGMENT_DURATION),
     // Album arts
     FFMPEGFS_OPT("--noalbumarts",                   m_noalbumarts, 1),
     FFMPEGFS_OPT("noalbumarts",                     m_noalbumarts, 1),
@@ -296,6 +305,9 @@ static struct fuse_opt ffmpegfs_opts[] =
     FFMPEGFS_OPT("decoding_errors=%u",              m_decoding_errors, 0),
     FFMPEGFS_OPT("--min_dvd_chapter_duration=%u",   m_min_dvd_chapter_duration, 0),
     FFMPEGFS_OPT("min_dvd_chapter_duration=%u",     m_min_dvd_chapter_duration, 0),
+    FFMPEGFS_OPT("--oldnamescheme=%u",              m_oldnamescheme, 0),
+    FFMPEGFS_OPT("oldnamescheme=%u",                m_oldnamescheme, 0),
+    // Experimental
     FFMPEGFS_OPT("--win_smb_fix=%u",                m_win_smb_fix, 0),
     FFMPEGFS_OPT("win_smb_fix=%u",                  m_win_smb_fix, 0),
     // FFmpegfs options
@@ -322,13 +334,14 @@ static struct fuse_opt ffmpegfs_opts[] =
 typedef std::map<std::string, AUTOCOPY, comp> AUTOCOPY_MAP;     /**< @brief Map command line option to AUTOCOPY enum */
 typedef std::map<std::string, PROFILE, comp> PROFILE_MAP;       /**< @brief Map command line option to PROFILE enum  */
 typedef std::map<std::string, PRORESLEVEL, comp> LEVEL_MAP;     /**< @brief Map command line option to LEVEL enum  */
+typedef std::map<std::string, RECODESAME, comp> RECODESAME_MAP; /**< @brief Map command line option to RECODESAME enum  */
 
 /**
   * List of AUTOCOPY options
   */
 static const AUTOCOPY_MAP autocopy_map =
 {
-    { "NONE",           AUTOCOPY_OFF },
+    { "OFF",            AUTOCOPY_OFF },
     { "MATCH",          AUTOCOPY_MATCH },
     { "MATCHLIMIT",     AUTOCOPY_MATCHLIMIT },
     { "STRICT",         AUTOCOPY_STRICT },
@@ -367,18 +380,33 @@ static const LEVEL_MAP level_map =
     { "HQ",             PRORESLEVEL_PRORES_HQ },
 };
 
+/**
+  * List if recode options.
+  */
+static const RECODESAME_MAP recode_map =
+{
+    // Recode to same format
+    { "NO",             RECODESAME_NO },
+    { "YES",            RECODESAME_YES },
+};
+
 static int          get_bitrate(const std::string & arg, BITRATE *bitrate);
 static int          get_samplerate(const std::string & arg, int *samplerate);
 static int          get_time(const std::string & arg, time_t *time);
 static int          get_size(const std::string & arg, size_t *size);
-static int          get_desttype(const std::string & arg, FFmpegfs_Format *video_format, FFmpegfs_Format *audio_format);
+static int          get_desttype(const std::string & arg, FFmpegfs_Format format[2]);
+static int          get_autocopy(const std::string & arg, AUTOCOPY *autocopy);
+static std::string  get_autocopy_text(AUTOCOPY autocopy);
 static int          get_autocopy(const std::string & arg, AUTOCOPY *autocopy);
 static std::string  get_autocopy_text(AUTOCOPY autocopy);
 static int          get_profile(const std::string & arg, PROFILE *profile);
 static std::string  get_profile_text(PROFILE profile);
 static int          get_level(const std::string & arg, PRORESLEVEL *level);
 static std::string  get_level_text(PRORESLEVEL level);
+static int          get_segment_duration(const std::string & arg, int64_t *value);
+static int          get_value(const std::string & arg, int *value);
 static int          get_value(const std::string & arg, std::string *value);
+static int          get_value(const std::string & arg, double *value);
 
 static int          ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_args *outargs);
 static bool         set_defaults(void);
@@ -746,11 +774,11 @@ static int get_size(const std::string & arg, size_t *size)
 /**
  * @brief Get destination type.
  * @param[in] arg - Format as string (MP4, OGG etc.).
- * @param[out] video_format - Selected video format.
- * @param[out] audio_format - Selected audio format.
+ * @param[out] format - Index 0: Selected video format.@n
+ * Index 1: Selected audio format.
  * @return Returns 0 if found; if not found returns -1.
  */
-int get_desttype(const std::string & arg, FFmpegfs_Format *video_format, FFmpegfs_Format * audio_format)
+static int get_desttype(const std::string & arg, FFmpegfs_Format format[2])
 {
     /** @todo: evaluate this function */
     size_t pos = arg.find('=');
@@ -762,7 +790,7 @@ int get_desttype(const std::string & arg, FFmpegfs_Format *video_format, FFmpegf
         if (results.size() > 0 && results.size() < 3)
         {
             // Check for valid destination type and obtain codecs and file type.
-            if (!video_format->init(results[0]))
+            if (!format[0].init(results[0]))
             {
                 std::fprintf(stderr, "INVALID PARAMETER: No codecs available for desttype: %s\n", results[0].c_str());
                 return 1;
@@ -770,7 +798,7 @@ int get_desttype(const std::string & arg, FFmpegfs_Format *video_format, FFmpegf
 
             if (results.size() == 2)
             {
-                if (!audio_format->init(results[1]))
+                if (!format[1].init(results[1]))
                 {
                     std::fprintf(stderr, "INVALID PARAMETER: No codecs available for desttype: %s\n", results[1].c_str());
                     return 1;
@@ -833,6 +861,53 @@ static std::string get_autocopy_text(AUTOCOPY autocopy)
     return "INVALID";
 }
 
+/**
+ * @brief Get recode option.
+ * @param[in] arg - One of the recode options.
+ * @param[out] recode - Upon return contains selected RECODESAME enum.
+ * @return Returns 0 if found; if not found returns -1.
+ */
+static int get_recodesame(const std::string & arg, RECODESAME *recode)
+{
+    size_t pos = arg.find('=');
+
+    if (pos != std::string::npos)
+    {
+        std::string data(arg.substr(pos + 1));
+
+        auto it = recode_map.find(data);
+
+        if (it == recode_map.end())
+        {
+            std::fprintf(stderr, "INVALID PARAMETER: Invalid recode option: %s\n", data.c_str());
+            return -1;
+        }
+
+        *recode = it->second;
+
+        return 0;
+    }
+
+    std::fprintf(stderr, "INVALID PARAMETER: Missing recode string\n");
+
+    return -1;
+}
+
+/**
+ * @brief Convert RECODESAME enum to human readable text.
+ * @param[in] recode - RECODESAME enum value to convert.
+ * @return RECODESAME enum as text or "INVALID" if not known.
+ */
+static std::string get_recodesame_text(RECODESAME recode)
+{
+    RECODESAME_MAP::const_iterator it = search_by_value(recode_map, recode);
+    if (it != recode_map.end())
+    {
+        return it->first;
+    }
+    return "INVALID";
+}
+
 /**
  * @brief Get profile option.
  * @param[in] arg - One of the auto profile options.
@@ -929,6 +1004,54 @@ static std::string get_level_text(PRORESLEVEL level)
     return "INVALID";
 }
 
+/**
+ * @brief Get HLS segment duration. Input value must be in seconds.
+ * @param[in] arg - One of the ProRes levels.
+ * @param[out] value - Upon return contains segment duration in AV_TIME_BASE units.
+ * @return Returns 0 if found; if not found returns -1.
+ */
+static int get_segment_duration(const std::string & arg, int64_t *value)
+{
+    double duration;
+    if (get_value(arg, &duration) < 0)
+    {
+        return -1;
+    }
+
+    if (*value <= 0)
+    {
+        std::fprintf(stderr, "INVALID PARAMETER: segment_duration %.1f is out of range. For obvious reasons this must be greater than zero.\n", duration);
+        return -1;
+    }
+
+    *value = static_cast<int>(duration * AV_TIME_BASE);
+
+    return 0;
+}
+
+/**
+ * @brief Get value form command line string.
+ * Finds whatever is after the "=" sign.
+ * @param[in] arg - Command line option.
+ * @param[in] value - Upon return, contains the value after the "=" sign.
+ * @return Returns 0 if found; if not found returns -1.
+ */
+static int get_value(const std::string & arg, int *value)
+{
+    size_t pos = arg.find('=');
+
+    if (pos != std::string::npos)
+    {
+        *value = atoi(arg.substr(pos + 1).c_str());
+
+        return 0;
+    }
+
+    std::fprintf(stderr, "INVALID PARAMETER: Missing value\n");
+
+    return -1;
+}
+
 /**
  * @brief Get value form command line string.
  * Finds whatever is after the "=" sign.
@@ -952,6 +1075,29 @@ static int get_value(const std::string & arg, std::string *value)
     return -1;
 }
 
+/**
+ * @brief Get value form command line string.
+ * Finds whatever is after the "=" sign.
+ * @param[in] arg - Command line option.
+ * @param[in] value - Upon return, contains the value after the "=" sign.
+ * @return Returns 0 if found; if not found returns -1.
+ */
+static int get_value(const std::string & arg, double *value)
+{
+    size_t pos = arg.find('=');
+
+    if (pos != std::string::npos)
+    {
+        *value = atof(arg.substr(pos + 1).c_str());
+
+        return 0;
+    }
+
+    std::fprintf(stderr, "INVALID PARAMETER: Missing value\n");
+
+    return -1;
+}
+
 /**
  * @brief FUSE option parsing function.
  * @param[in] data - is the user data passed to the fuse_opt_parse() function
@@ -1051,12 +1197,16 @@ static int ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
     }
     case KEY_DESTTYPE:
     {
-        return get_desttype(arg, &params.m_format[0], &params.m_format[1]);
+        return get_desttype(arg, params.m_format);
     }
     case KEY_AUTOCOPY:
     {
         return get_autocopy(arg, &params.m_autocopy);
     }
+    case KEY_RECODESAME:
+    {
+        return get_recodesame(arg, &params.m_recodesame);
+    }
     case KEY_PROFILE:
     {
         return get_profile(arg, &params.m_profile);
@@ -1073,6 +1223,10 @@ static int ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
     {
         return get_samplerate(arg, &params.m_audiosamplerate);
     }
+    case KEY_AUDIO_CHANNELS:
+    {
+        return get_value(arg, &params.m_audiochannels);
+    }
     case KEY_SCRIPTFILE:
     {
         return get_value(arg, &params.m_scriptfile);
@@ -1085,6 +1239,10 @@ static int ffmpegfs_opt_proc(void* data, const char* arg, int key, struct fuse_a
     {
         return get_bitrate(arg, &params.m_videobitrate);
     }
+    case KEY_SEGMENT_DURATION:
+    {
+        return get_segment_duration(arg, &params.m_segment_duration);
+    }
     case KEY_EXPIRY_TIME:
     {
         return get_time(arg, &params.m_expiry_time);
@@ -1156,94 +1314,56 @@ static void print_params(void)
 
     transcoder_cache_path(cachepath);
 
-    Logging::trace(nullptr, PACKAGE_NAME " options:\n\n"
-                                         "Base Path         : %1\n"
-                                         "Mount Path        : %2\n\n"
-                                         "Smart Transcode   : %3\n"
-                                         "Auto Copy         : %4\n"
-                                         "Audio File Type   : %5\n"
-                                         "Video File Type   : %6\n"
-                                         "Profile           : %7\n"
-                                         "Level             : %8\n"
-                                         "\nAudio\n\n"
-                                         "Audio Codecs      : %9+%10\n"
-                                         "Audio Bitrate     : %11\n"
-                                         "Audio Sample Rate : %12\n"
-                                         "\nVideo\n\n"
-                                         "Video Size/Pixels : width=%13 height=%14\n"
-                                         "Deinterlace       : %15\n"
-                                         "Remove Album Arts : %16\n"
-                                         "Video Codec       : %17\n"
-                                         "Video Bitrate     : %18\n"
-                                         "\nVirtual Script\n\n"
-                                         "Create script     : %19\n"
-                                         "Script file name  : %20\n"
-                                         "Input file        : %21\n"
-                                         "\nLogging\n\n"
-                                         "Max. Log Level    : %22\n"
-                                         "Log to stderr     : %23\n"
-                                         "Log to syslog     : %24\n"
-                                         "Logfile           : %25\n"
-                                         "\nCache Settings\n\n"
-                                         "Expiry Time       : %26\n"
-                                         "Inactivity Suspend: %27\n"
-                                         "Inactivity Abort  : %28\n"
-                                         "Pre-buffer size   : %29\n"
-                                         "Max. Cache Size   : %20\n"
-                                         "Min. Disk Space   : %31\n"
-                                         "Cache Path        : %32\n"
-                                         "Disable Cache     : %33\n"
-                                         "Maintenance Timer : %34\n"
-                                         "Clear Cache       : %35\n"
-                                         "\nVarious Options\n\n"
-                                         "Max. Threads      : %36\n"
-                                         "Decoding Errors   : %37\n"
-                                         "Min. DVD chapter  : %38\n"
-                                         "\nExperimental Options\n\n"
-                                         "Windows 10 Fix    : %39\n",
-                   params.m_basepath.c_str(),
-                   params.m_mountpath.c_str(),
-                   params.smart_transcode() ? "yes" : "no",
-                   get_autocopy_text(params.m_autocopy).c_str(),
-                   params.m_format[1].desttype().c_str(),
-            params.m_format[0].desttype().c_str(),
-            get_profile_text(params.m_profile).c_str(),
-            get_level_text(params.m_level).c_str(),
-            get_codec_name(params.m_format[0].audio_codec_id(), true),
-            get_codec_name(params.m_format[1].audio_codec_id(), true),
-            format_bitrate(params.m_audiobitrate).c_str(),
-            format_samplerate(params.m_audiosamplerate).c_str(),
-            format_number(params.m_videowidth).c_str(),
-            format_number(params.m_videoheight).c_str(),
-        #ifndef USING_LIBAV
-            params.m_deinterlace ? "yes" : "no",
-        #else
-            "not supported",
-        #endif  // !USING_LIBAV
-            params.m_noalbumarts ? "yes" : "no",
-            get_codec_name(params.m_format[0].video_codec_id(), true),
-            format_bitrate(params.m_videobitrate).c_str(),
-            params.m_enablescript ? "yes" : "no",
-            params.m_scriptfile.c_str(),
-            params.m_scriptsource.c_str(),
-            params.m_log_maxlevel.c_str(),
-            params.m_log_stderr ? "yes" : "no",
-            params.m_log_syslog ? "yes" : "no",
-            !params.m_logfile.empty() ? params.m_logfile.c_str() : "none",
-            format_time(params.m_expiry_time).c_str(),
-            format_time(params.m_max_inactive_suspend).c_str(),
-            format_time(params.m_max_inactive_abort).c_str(),
-            format_size(params.m_prebuffer_size).c_str(),
-            format_size(params.m_max_cache_size).c_str(),
-            format_size(params.m_min_diskspace).c_str(),
-            cachepath.c_str(),
-            params.m_disable_cache ? "yes" : "no",
-            params.m_cache_maintenance ? format_time(params.m_cache_maintenance).c_str() : "inactive",
-            params.m_clear_cache ? "yes" : "no",
-            format_number(params.m_max_threads).c_str(),
-            params.m_decoding_errors ? "break transcode" : "ignore",
-            format_duration(params.m_min_dvd_chapter_duration * AV_TIME_BASE).c_str(),
-            params.m_win_smb_fix ? "inactive" : "SMB Lockup Fix Active");
+    Logging::trace(nullptr, "********* " PACKAGE_NAME " Options *********");
+    Logging::trace(nullptr, "Base Path         : %1", params.m_basepath.c_str());
+    Logging::trace(nullptr, "Mount Path        : %1", params.m_mountpath.c_str());
+    Logging::trace(nullptr, "Smart Transcode   : %1", params.smart_transcode() ? "yes" : "no");
+    Logging::trace(nullptr, "Auto Copy         : %1", get_autocopy_text(params.m_autocopy).c_str());
+    Logging::trace(nullptr, "Recode to same fmt: %1", get_recodesame_text(params.m_recodesame).c_str());
+    Logging::trace(nullptr, "Audio File Type   : %1", params.m_format[1].desttype().c_str());
+    Logging::trace(nullptr, "Video File Type   : %1", params.m_format[0].desttype().c_str());
+    Logging::trace(nullptr, "Profile           : %1", get_profile_text(params.m_profile).c_str());
+    Logging::trace(nullptr, "Level             : %1", get_level_text(params.m_level).c_str());
+    Logging::trace(nullptr, "--------- Audio ---------");
+    Logging::trace(nullptr, "Codecs            : %1+%2", get_codec_name(params.m_format[0].audio_codec_id(), true), get_codec_name(params.m_format[1].audio_codec_id(), true));
+    Logging::trace(nullptr, "Bitrate           : %1", format_bitrate(params.m_audiobitrate).c_str());
+    Logging::trace(nullptr, "Sample Rate       : %1", format_samplerate(params.m_audiosamplerate).c_str());
+    Logging::trace(nullptr, "Max. Channels     : %1", params.m_audiochannels);
+    Logging::trace(nullptr, "--------- Video ---------");
+    Logging::trace(nullptr, "Dimension         : width=%1 height=%2", format_number(params.m_videowidth).c_str(), format_number(params.m_videoheight).c_str());
+    Logging::trace(nullptr, "Deinterlace       : %1", params.m_deinterlace ? "yes" : "no");
+    Logging::trace(nullptr, "Codec             : %1", get_codec_name(params.m_format[0].video_codec_id(), true));
+    Logging::trace(nullptr, "Bitrate           : %1", format_bitrate(params.m_videobitrate).c_str());
+    Logging::trace(nullptr, "--------- HLS Options ---------");
+    Logging::trace(nullptr, "Segment Duration  : %1", format_time(params.m_segment_duration).c_str());
+    Logging::trace(nullptr, "--------- Virtual Script ---------");
+    Logging::trace(nullptr, "Create script     : %1", params.m_enablescript ? "yes" : "no");
+    Logging::trace(nullptr, "Script file name  : %1", params.m_scriptfile.c_str());
+    Logging::trace(nullptr, "Input file        : %1", params.m_scriptsource.c_str());
+    Logging::trace(nullptr, "--------- Logging ---------");
+    Logging::trace(nullptr, "Max. Log Level    : %1", params.m_log_maxlevel.c_str());
+    Logging::trace(nullptr, "Log to stderr     : %1", params.m_log_stderr ? "yes" : "no");
+    Logging::trace(nullptr, "Log to syslog     : %1", params.m_log_syslog ? "yes" : "no");
+    Logging::trace(nullptr, "Logfile           : %1", !params.m_logfile.empty() ? params.m_logfile.c_str() : "none");
+    Logging::trace(nullptr, "--------- Cache Settings ---------");
+    Logging::trace(nullptr, "Expiry Time       : %1", format_time(params.m_expiry_time).c_str());
+    Logging::trace(nullptr, "Inactivity Suspend: %1", format_time(params.m_max_inactive_suspend).c_str());
+    Logging::trace(nullptr, "Inactivity Abort  : %1", format_time(params.m_max_inactive_abort).c_str());
+    Logging::trace(nullptr, "Pre-buffer size   : %1", format_size(params.m_prebuffer_size).c_str());
+    Logging::trace(nullptr, "Max. Cache Size   : %1", format_size(params.m_max_cache_size).c_str());
+    Logging::trace(nullptr, "Min. Disk Space   : %1", format_size(params.m_min_diskspace).c_str());
+    Logging::trace(nullptr, "Cache Path        : %1", cachepath.c_str());
+    Logging::trace(nullptr, "Disable Cache     : %1", params.m_disable_cache ? "yes" : "no");
+    Logging::trace(nullptr, "Maintenance Timer : %1", params.m_cache_maintenance ? format_time(params.m_cache_maintenance).c_str() : "inactive");
+    Logging::trace(nullptr, "Clear Cache       : %1", params.m_clear_cache ? "yes" : "no");
+    Logging::trace(nullptr, "--------- Various Options ---------");
+    Logging::trace(nullptr, "Remove Album Arts : %1", params.m_noalbumarts ? "yes" : "no");
+    Logging::trace(nullptr, "Max. Threads      : %1", format_number(params.m_max_threads).c_str());
+    Logging::trace(nullptr, "Decoding Errors   : %1", params.m_decoding_errors ? "break transcode" : "ignore");
+    Logging::trace(nullptr, "Min. DVD Chapter  : %1", format_duration(params.m_min_dvd_chapter_duration * AV_TIME_BASE).c_str());
+    Logging::trace(nullptr, "Old Name Scheme   : %1", params.m_oldnamescheme ? "yes" : "no");
+    Logging::trace(nullptr, "--------- Experimental Options ---------");
+    Logging::trace(nullptr, "Windows 10 Fix    : %1", params.m_win_smb_fix ? "inactive" : "SMB Lockup Fix Active");
 }
 
 /**
@@ -1281,10 +1401,8 @@ int main(int argc, char *argv[])
 #if !LAVC_DEP_AV_FILTER_REGISTER
     avfilter_register_all();
 #endif // LAVC_DEP_AV_FILTER_REGISTER
-#ifndef USING_LIBAV
     // Redirect FFmpeg logs
     av_log_set_callback(ffmpeg_log);
-#endif
 
     // Set default
     params.m_max_threads = static_cast<unsigned int>(get_nprocs() * 16);
