@@ -44,11 +44,50 @@
 #include <sys/stat.h>
 #include <string>
 #include <vector>
+#include <map>
+
+// Disable annoying warnings outside our code
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wconversion"
+#pragma GCC diagnostic ignored "-Wsign-conversion"
+#ifdef __cplusplus
+extern "C" {
+#endif
+extern "C" {
+#include <libavutil/avutil.h>
+}
+#ifdef __cplusplus
+}
+#endif
+#pragma GCC diagnostic pop
+
+#pragma pack(push, 1)
+
+#define IMAGE_FRAME_TAG         "IMGFRAME"      /**< @brief Tag of an image frame header for the frame images buffer. */
+/**
+  * @brief Image frame header
+  *
+  * This image frame header will always start at a 8K boundary of the cache.
+  * It can be used to find the next image by seeking a 8K block starting with tag.
+  */
+typedef struct IMAGE_FRAME
+{
+    char            m_tag[8];                   /**< @brief Start tag, always ascii "IMGFRAME". */
+    uint32_t        m_frame_no;                 /**< @brief Number of the frame image. 0 if not yet decoded. */
+    uint64_t        m_offset;                   /**< @brief Offset in index file. */
+    uint32_t        m_size;                     /**< @brief Image size in bytes. */
+    uint8_t         m_reserved[8];              /**< @brief Reserved. Pad structure to 32 bytes. */
+    // ...data
+} IMAGE_FRAME;
+#pragma pack(pop)
+typedef IMAGE_FRAME const *LPCIMAGE_FRAME;      /**< @brief Pointer version of IMAGE_FRAME */
+typedef IMAGE_FRAME *LPIMAGE_FRAME;             /**< @brief Pointer to const version of IMAGE_FRAME */
 
 /** @brief Virtual file types enum
  */
 typedef enum VIRTUALTYPE
 {
+    VIRTUALTYPE_PASSTHROUGH,                                        /**< @brief passthrough file, not used */
     VIRTUALTYPE_DISK,                                               /**< @brief Regular disk file to transcode */
     VIRTUALTYPE_SCRIPT,                                             /**< @brief Virtual script */
 #ifdef USE_LIBVCD
@@ -66,20 +105,33 @@ typedef enum VIRTUALTYPE
 typedef VIRTUALTYPE const *LPCVIRTUALTYPE;                          /**< @brief Pointer version of VIRTUALTYPE */
 typedef VIRTUALTYPE LPVIRTUALTYPE;                                  /**< @brief Pointer to const version of VIRTUALTYPE */
 
+#define VIRTUALFLAG_NONE            0x00000000                      /**< @brief No flags */
+#define VIRTUALFLAG_PASSTHROUGH     0x00000001                      /**< @brief passthrough file, not used */
+#define VIRTUALFLAG_DIRECTORY       0x00000002                      /**< @brief File is a virtual directory */
+#define VIRTUALFLAG_FILESET         0x00000004                      /**< @brief File is file set (images, HLS) */
+#define VIRTUALFLAG_FRAME           0x00000008                      /**< @brief File is part of a set of frames */
+#define VIRTUALFLAG_HLS             0x00000010                      /**< @brief File is part of a set of HLS transport stream (ts) files */
+
 /** @brief Virtual file definition
  */
 typedef struct VIRTUALFILE
 {
     VIRTUALFILE()
         : m_type(VIRTUALTYPE_DISK)
+        , m_flags(VIRTUALFLAG_NONE)
         , m_format_idx(0)
         , m_full_title(false)
         , m_duration(0)
+        , m_predicted_size(0)
+        , m_video_frame_count(0)
     {
 
     }
 
+    uint32_t get_segment_count() const;
+
     VIRTUALTYPE         m_type;                                     /**< @brief Type of this virtual file */
+    int                 m_flags;                                    /**< @brief One of the VIRTUALFLAG_* flags */
 
     int                 m_format_idx;                               /**< @brief Index into params.format[] array */
     std::string         m_origfile;                                 /**< @brief Sanitised original file name */
@@ -87,6 +139,8 @@ typedef struct VIRTUALFILE
 
     bool                m_full_title;                               /**< @brief If true, ignore m_chapter_no and provide full track */
     int64_t             m_duration;                                 /**< @brief Track/chapter duration, in AV_TIME_BASE fractional seconds. */
+    size_t              m_predicted_size;                           /**< @brief Use this as the size instead of computing it over and over. */
+    uint32_t            m_video_frame_count;                        /**< @brief Number of frames in video or 0 if not a video */
 
     std::vector<char>   m_file_contents;                            /**< @brief Buffer for virtual files */
 
@@ -244,7 +298,7 @@ public:
      * @brief Get virtual file object
      * @return Current virtual file object or nullptr if unset.
      */
-    LPCVIRTUALFILE      virtualfile() const;
+    LPVIRTUALFILE       virtualfile();
     /**
      * @brief Get source filename.
      * @return Returns source filename.
@@ -260,11 +314,11 @@ protected:
     /** @brief Set the virtual file object.
      * @param[in] virtualfile - LPCVIRTUALFILE of file to set.
      */
-    void                set_virtualfile(LPCVIRTUALFILE virtualfile);
+    void                set_virtualfile(LPVIRTUALFILE virtualfile);
 
 private:
     std::string         m_path;                                     /**< @brief Source path (directory without file name) */
-    LPCVIRTUALFILE      m_virtualfile;                              /**< @brief Virtual file object of current file */
+    LPVIRTUALFILE       m_virtualfile;                              /**< @brief Virtual file object of current file */
 };
 
 #endif // FILEIO_H
